import { BaseConnection, WsConnection } from "tsrpc";
import { server } from "../..";
import { RandomEx } from "../../Engine/Utils/Number/RandomEx";
import { ServiceType } from "../../shared/protocols/serviceProto";


/**
 * Room
 */
export default class Room {

	//#region static

	private static maxRoomId: number = 0;
	private static rooms: { [roomId: number]: Room } = {};

	//#endregion

	//#region public

	public RoomId: number = 0;

	//#endregion

	//#region private

	private conns: BaseConnection[] = [];

	private nowPlayer: [index: number, name: string] = undefined;

	//#endregion

	//#region Lifecycle

	/**
	 *
	 */
	constructor() {
		// 给每个新房间生成一个唯一的 ID
		this.RoomId = ++Room.maxRoomId;
		Room.rooms[this.RoomId] = this;

		const now: number = RandomEx.GetInt(0, 2);
	}

	//#endregion

	//#region Custom

	/** GetRoom */
	public static GetRoom(roomId: number): Room {
		return this.rooms[roomId];
	}

	//#endregion

	//#region Custom

	/** Join */
	public Join(conn: BaseConnection): void {
		this.conns.push(conn);
	}

	/** Exit */
	public Exit(conn: BaseConnection): void {
		server.broadcastMsg("room/Exit", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** GotoGame */
	public GotoGame(): void {
		server.broadcastMsg("room/GoToGame", 0, <WsConnection<ServiceType>[]>this.conns);
	}

	/** ConnCount */
	public ConnCount(): number {
		return this.conns.length;
	}

	//#endregion
}
